package com.winhc.data.push.controller;

import cn.hutool.core.lang.Assert;
import com.aliyun.openservices.shade.org.apache.commons.lang3.StringUtils;
import com.winhc.data.push.bean.TaskRunParam;
import com.winhc.data.push.common.TaskEnum;
import com.winhc.data.push.common.TaskRunMap;
import com.winhc.data.push.constant.BaseParam;
import com.winhc.data.push.service.SynDataService;
import com.winhc.data.push.utils.BaseUtils;
import com.winhc.data.push.vo.ResponseVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.bson.Document;
import org.jetbrains.annotations.NotNull;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import static com.winhc.data.push.common.TaskRunMap.TASK_ARGS;
import static com.winhc.data.push.constant.BaseParam.SYN_GW_UPLOAD_TASKS;
import static com.winhc.data.push.constant.BaseParam.SYN_TC_UPLOAD_TASKS;
import static com.winhc.data.push.utils.BaseUtils.getObjectId;


/**
 * @author π
 * @Description :国网推送数据
 * @date 2022/7/20 16:24
 */
@Slf4j
@AllArgsConstructor
@RestController
@Api(tags = "任务触发gw", value = "task/gw")
@RequestMapping("task/gw")
public class GwPushDataController {

    private final SynDataService synDataService;


    @ApiOperation(value = "触发任务")
    @GetMapping("push/{ds}")
    public ResponseVo touch(@PathVariable String ds, @RequestParam String tns, @RequestParam String sign) {
        //日期是昨天之前数据
        List<String> out_tns = getList(ds, tns, sign);
        long start = System.currentTimeMillis();
        try {
            TaskRunParam runParam = TASK_ARGS.get("gw").getTaskRunParam();
            String table = runParam.getSyn_upload_tasks();
            String id = getObjectId(ds, runParam.getCustomer_name());
            String re = synDataService.saveTask(table, out_tns, ds, id);
            return ResponseVo.success(start, re);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return ResponseVo.failure(start, e.getMessage());
        }
    }

    @ApiOperation(value = "强制触发任务")
    @GetMapping("repush/{ds}")
    public ResponseVo reTouch(@PathVariable String ds, @RequestParam String tns, @RequestParam String sign) {
        List<String> out_tns = getList(ds, tns, sign);

        long start = System.currentTimeMillis();
        try {
            Document d = new Document();
            d.put("update_time", new Date());
            d.put("status", TaskEnum.STATUS.CREATE.code);
            d.put("ds", ds);
            d.put("tns", out_tns);
            TaskRunParam runParam = TASK_ARGS.get("gw").getTaskRunParam();
            String table = runParam.getSyn_upload_tasks();
            String id = getObjectId(ds, runParam.getCustomer_name());
            Boolean re = synDataService.updateTaskStatus(table, d, id);
            return ResponseVo.success(start, re ? "restart" : "error");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return ResponseVo.failure(start, e.getMessage());
        }
    }

    @NotNull
    public static List<String> getList(String ds, String tns, String sign) {
        Assert.isTrue(BaseUtils.verifyDate(ds), "ds 不合法");
        Assert.isTrue(BaseParam.sign.equals(sign), "验签不通过！");
        Assert.isTrue(StringUtils.isNotBlank(tns), "参数不能为空!");
        List<String> out_tns = tns.equals("all") ? new ArrayList<>() : Arrays.asList(tns.split(","));
        return out_tns;
    }


}
