package com.winhc.data.push.service.impl;

import com.aliyun.odps.Column;
import com.aliyun.odps.Instance;
import com.aliyun.odps.Odps;
import com.aliyun.odps.data.Record;
import com.aliyun.odps.task.SQLTask;
import com.aliyun.openservices.shade.org.apache.commons.lang3.StringUtils;
import com.winhc.data.push.service.OdpsService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author π
 * @Description:
 * @date 2022/7/24 19:08
 */
@Slf4j
@Service
@AllArgsConstructor
public class OdpsServiceImpl implements OdpsService {

    private final Odps odps;

    @Override
    public List<String> query(String sql, String ds) {
        Instance i;
        try {
            i = SQLTask.run(odps, sql.replaceAll("@ds", ds));
            i.waitForSuccess();
            List<Record> records = SQLTask.getResult(i);
            List<String> ds_list = records.
                    stream()
                    .map(x -> x.get(0).toString().split("tn=")[1])
                    .distinct()
                    .collect(Collectors.toList());
            if (!records.isEmpty()) {
                String head = Arrays.stream(records.get(0).getColumns()).map(Column::getName).findFirst().orElse("");
                if (StringUtils.isNotBlank(head)) {
                    ds_list.add(head.split("tn=")[1]);
                }
            }
            return ds_list;
        } catch (Exception e) {
            e.printStackTrace();
            return new ArrayList<>();
        }
    }
}
