package com.winhc.data.push.service.impl;

import cn.cocowwy.dingtalk.DingTalkGroupApi;
import com.alibaba.fastjson.JSON;
import com.aliyun.openservices.shade.org.apache.commons.lang3.StringUtils;
import com.aliyun.oss.OSS;
import com.aliyun.oss.model.CopyObjectResult;
import com.aliyun.oss.model.PutObjectRequest;
import com.mongodb.client.FindIterable;
import com.mongodb.client.MongoDatabase;
import com.mongodb.client.model.FindOneAndUpdateOptions;
import com.mongodb.client.model.UpdateOptions;
import com.winhc.data.push.bean.TaskRunParam;
import com.winhc.data.push.common.Constant;
import com.winhc.data.push.common.TaskEnum;
import com.winhc.data.push.configuration.OSSAccessProperties;
import com.winhc.data.push.framework.MongoDbFastScan;
import com.winhc.data.push.service.SynDataService;
import com.winhc.data.push.service.TouchService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.bson.Document;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.stereotype.Service;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.util.*;
import java.util.function.Consumer;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

import static cn.hutool.crypto.SecureUtil.md5;
import static com.mongodb.client.model.Filters.*;
import static com.winhc.data.push.common.Constant.*;
import static com.winhc.data.push.constant.BaseParam.*;
import static com.winhc.data.push.utils.BaseUtils.getObjectId;
import static com.winhc.data.push.utils.BaseUtils.getPrcessDate;
import static com.winhc.data.push.utils.DateUtils.getPrcessDateV2;
import static com.winhc.data.push.utils.OssUtils.existsBucketName;
import static com.winhc.data.push.utils.OssUtils.listKeys;
import static com.winhc.data.push.utils.TransToExcelNewCompany.dataPathPrefix;
import static com.winhc.data.push.utils.TransToExcelNewCompany.run;

/**
 * @author π
 * @Description:
 * @date 2022/7/20 17:02
 */
@Slf4j
@Service
@AllArgsConstructor
public class SynDataServiceImpl implements SynDataService {

    private final MongoTemplate mongoTemplate;
    private final OSS ossClient;
    private final OSSAccessProperties project;
    private final TouchService touchService;
    private final DingTalkGroupApi dingTalkApi;


    @Override
    public void push(String tn, String ds) {
        String id = tn + "_" + ds;
        try {
            updateTask(tn, ds, "running");
            sendMessage("task running !!! " + id);
            String project = "winhc_ng";
            if (touchService.export2Mongo(project, ds)) {
                copyOSS(ds);
                if (touchService.export2OSS(project, ds)) {
                    saveObject(ds);
                    updateTask(tn, ds, "success");
                    sendMessage("导出 任务 成功 !!! " + id);
                    saveEmptyObject(ds);
                } else {
                    log.error("导出OSS 失败 !!!");
                    sendMessage("导出 OSS 失败!!! " + id);
                    updateTask(tn, ds, "oss copy error");
                }
            } else {
                log.error("导出Mongo 失败!!!");
                sendMessage("导出 Mongo 失败!!! " + id);
                updateTask(tn, ds, "export mongo error");
            }

        } catch (Exception e) {
            sendMessage("任务失败!!! " + id + " |  error " + e.getMessage());
            updateTask(tn, ds, "task error");
            e.printStackTrace();
            log.error("export error: {}", e.getMessage());
        }
    }

    @Override
    public void pushTc(String ds) {
        this.pushTc(null, ds);
    }


    /**
     * 同城数据输出
     *
     * @param tns
     * @param ds
     */
    @Override
    public void pushTc(List<String> tns, String ds) {
        String id = getObjectId(ds);
        try {
            updateTaskStatus(SYN_TC_UPLOAD_TASKS, ds, TaskEnum.STATUS.RUNNING.code, id);
            sendMessage("task running !!! " + id);
            String project = "winhc_ng";
            if (touchService.export2OSSTc(project, ds, tns)) {
                saveObject(ds, TC_BUCKET_NAME, TC_OSS_PRE);
                updateTaskStatus(SYN_TC_UPLOAD_TASKS, ds, TaskEnum.STATUS.SUCCESS.code, id);
                sendMessage("导出 任务 成功 !!! " + id);
            } else {
                log.error("导出OSS 失败 !!!");
                sendMessage("导出 OSS 失败!!! " + id);
                updateTaskStatus(SYN_TC_UPLOAD_TASKS, ds, TaskEnum.STATUS.ERROR.code, id);
            }

        } catch (Exception e) {
            sendMessage("任务失败!!! " + id + " |  error " + e.getMessage());
            updateTaskStatus(SYN_TC_UPLOAD_TASKS, ds, TaskEnum.STATUS.TASK_ERROR.code, id);
            e.printStackTrace();
            log.error("export error: {}", e.getMessage());
        }
    }

    /**
     * 国网数据输出
     *
     * @param tns
     * @param ds
     */
    @Override
    public void pushGw(List<String> tns, String ds, TaskRunParam runParam) {
        //String id = ds + "_" + runParam.getCustomer_name();
        String id = getObjectId(ds, runParam.getCustomer_name());
        try {
            updateTaskStatus(runParam.getSyn_upload_tasks(), ds, TaskEnum.STATUS.RUNNING.code, id);
            sendMessage("task running !!! " + id + "\n" + runParam.getSource_table());
            String project = "winhc_ng";
            if (touchService.export2OSSGw(project, ds, tns, runParam)) {
                //保存集合
                saveObjectBase(ds, runParam);
                updateTaskStatus(runParam.getSyn_upload_tasks(), ds, TaskEnum.STATUS.SUCCESS.code, id);
                sendMessage("导出 任务 成功 !!! " + id + "\n" + runParam.getSource_table());
            } else {
                log.error("导出OSS 失败 !!!");
                sendMessage("导出 OSS 失败!!! " + id + "\n" + runParam.getSource_table());
                updateTaskStatus(runParam.getSyn_upload_tasks(), ds, TaskEnum.STATUS.ERROR.code, id);
            }

        } catch (Exception e) {
            sendMessage("任务失败!!! " + id + " |  error " + e.getMessage() + "\n" + runParam.getSource_table());
            updateTaskStatus(runParam.getSyn_upload_tasks(), ds, TaskEnum.STATUS.TASK_ERROR.code, id);
            e.printStackTrace();
            log.error("export error: {}", e.getMessage());
        }
    }

    /**
     * 安硕数据工商数据输出
     *
     * @param tns
     * @param ds
     */
    @Override
    public void pushAnshuoCompany(List<String> tns, String ds) {
        String id = ds;
        String message_pre = "安硕工商推送 ";
        try {
            updateTaskStatus(SYN_ANSHUO_GONGSHANG_TASKS, ds, TaskEnum.STATUS.RUNNING.code, id);
            sendMessage(message_pre + "task running !!! " + id);
            String project = "winhc_ng";
            if (touchService.export2OSSAnshuoCompany(project, ds, tns)) {
                //待修改
                saveObjectAnshuo(ds, ANSHUO_BUCKET_NAME, ANSHUO_OSS_PRE);
                updateTaskStatus(SYN_ANSHUO_GONGSHANG_TASKS, ds, TaskEnum.STATUS.SUCCESS.code, id);
                sendMessage(message_pre + "导出 任务 成功 !!! " + id);
            } else {
                log.error(message_pre + "导出OSS 失败 !!!");
                sendMessage(message_pre + "导出 OSS 失败!!! " + id);
                updateTaskStatus(SYN_ANSHUO_GONGSHANG_TASKS, ds, TaskEnum.STATUS.ERROR.code, id);
            }
        } catch (Exception e) {
            sendMessage(message_pre + "任务失败!!! " + id + " |  error " + e.getMessage());
            updateTaskStatus(SYN_ANSHUO_GONGSHANG_TASKS, ds, TaskEnum.STATUS.TASK_ERROR.code, id);
            e.printStackTrace();
            log.error(message_pre + "export error: {}", e.getMessage());
        }
    }


    @Override
    public String saveTask(String tn, String ds) {
        String id = tn + "_" + ds;
        if (findTaskExists(new Document("_id", id))) {
            return "task repeat";
        } else {
            Document d = new Document();
            d.put("_id", id);
            d.put("create_time", new Date());
            d.put("update_time", new Date());
            d.put("status", "create");
            d.put("tn", tn);
            d.put("ds", ds);
            mongoTemplate.getCollection(SYN_ANSHUO_TASKS).insertOne(d);
            return "task create success";
        }

    }

    @Override
    public String saveTask(String collectionName, List<String> tns, String ds) {
        String id = getObjectId(ds);
        if (findTaskExists(collectionName, new Document("_id", id))) {
            return "task repeat";
        } else {
            if (tns == null || tns.size() == 0) {
                tns = new ArrayList<>();
            }
            Document d = new Document();
            d.put("_id", id);
            d.put("create_time", new Date());
            d.put("update_time", new Date());
            d.put("status", "create");
            d.put("tns", tns);
            d.put("ds", ds);
            mongoTemplate.getCollection(collectionName).insertOne(d);
            return "task create success";
        }

    }

    @Override
    public String saveTask(String collectionName, List<String> tns, String ds, String id) {
        if (findTaskExists(collectionName, new Document("_id", id))) {
            return "task repeat";
        } else {
            if (tns == null || tns.size() == 0) {
                tns = new ArrayList<>();
            }
            Document d = new Document();
            d.put("_id", id);
            d.put("create_time", new Date());
            d.put("update_time", new Date());
            d.put("status", "create");
            d.put("tns", tns);
            d.put("ds", ds);
            mongoTemplate.getCollection(collectionName).insertOne(d);
            return "task create success";
        }

    }

    @Override
    public void sendMessage(String message) {
        try {
            dingTalkApi.sendTextByPhones("data_push_message", "winhc\n" + message, Collections.singletonList("17602140784"));
        } catch (Exception e) {
            log.error("send message error : {}", e.getMessage());
        }

    }

    @Override
    public Boolean findTaskExists(Document doc) {
        return mongoTemplate.getCollection(SYN_ANSHUO_TASKS).find(doc).iterator().hasNext();
    }

    @Override
    public Boolean findTaskExists(String collectionName, Document doc) {
        return mongoTemplate.getCollection(collectionName).find(doc).iterator().hasNext();
    }

    @Override
    public List<Document> findAllTask(Document doc) {
        FindIterable<Document> it = mongoTemplate.getCollection(SYN_ANSHUO_TASKS).find(doc);
        return StreamSupport.stream(it.spliterator(), false).collect(Collectors.toList());
    }


    @Override
    public List<Document> findAllTask(String collectionName, Document doc) {
        FindIterable<Document> it = mongoTemplate.getCollection(collectionName).find(doc);
        return StreamSupport.stream(it.spliterator(), false).collect(Collectors.toList());
    }


    @Override
    public Boolean updateTask(String tn, String ds, String status) {
        String _id = tn + "_" + ds;
        Document d = new Document();
        d.put("update_time", new Date());
        d.put("status", status);
        d.put("tn", tn);
        d.put("ds", ds);
        mongoTemplate.getCollection(SYN_ANSHUO_TASKS)
                .findOneAndUpdate(eq("_id", _id), new Document("$set", d), new FindOneAndUpdateOptions().upsert(true));
        return true;
    }

    @Override
    public Boolean updateTaskStatus(String collectionName, String ds, String status) {
        return this.updateTaskStatus(collectionName, ds, status, ds);
    }


    @Override
    public Boolean updateTaskStatus(String collectionName, Document d, String id) {
        mongoTemplate.getCollection(collectionName)
                .findOneAndUpdate(eq("_id", id), new Document("$set", d), new FindOneAndUpdateOptions().upsert(true));
        return true;
    }

    @Override
    public Boolean updateTaskStatus(String collectionName, String ds, String status, String id) {
        Document d = new Document();
        d.put("update_time", new Date());
        d.put("status", status);
        d.put("ds", ds);
        mongoTemplate.getCollection(collectionName)
                .findOneAndUpdate(eq("_id", id), new Document("$set", d), new FindOneAndUpdateOptions().upsert(true));
        return true;
    }

    @Override
    public String uploadTask(String ds) {
        if (findTaskExists(SYN_ANSHUO_UPLOAD_TASKS, new Document("_id", ds))) {
            return "task repeat";
        } else {
            Document d = new Document();
            d.put("_id", ds);
            d.put("create_time", new Date());
            d.put("update_time", new Date());
            d.put("status", TaskEnum.STATUS.CREATE.code);
            d.put("ds", ds);
            mongoTemplate.getCollection(SYN_ANSHUO_UPLOAD_TASKS).insertOne(d);
            return "task create success";
        }
    }

    @Override
    public void uploadToOss(String ds) {
        try {
            sendMessage("开始安硕每日任务 uploadToOss , 数据分区 : " + ds + " !!! ");
            String objectName = run(ds);
            updateTaskStatus(SYN_ANSHUO_UPLOAD_TASKS, ds, TaskEnum.STATUS.SUCCESS.code);
            String downLoadPath = "https://data-exchange-as.oss-cn-shanghai.aliyuncs.com/" + objectName;
            String zipSize = FileUtils.byteCountToDisplaySize(FileUtils.sizeOf(new File(dataPathPrefix + ds + ".zip")));
            sendMessage("安硕每日任务上传oss成功,数据分区 : " + ds + " !!! " + "大小 : " + zipSize + " . 下载地址 : \n" + downLoadPath);
        } catch (Exception e) {
            updateTaskStatus(SYN_ANSHUO_UPLOAD_TASKS, ds, TaskEnum.STATUS.ERROR.code);
            log.error("uploadToOss error : {}", e.getMessage());
            sendMessage("安硕每日任务 uploadToOss 失败 , error : " + e.getMessage());
        }

    }

    private void copyOSS(String ds) {
        //清空七天之前数据
        long lastWeek = Long.parseLong(ds) - 7;
        long deletedCount = mongoTemplate.getCollection("xf_as_upload_oss_log")
                .deleteMany(and(lt("ds", lastWeek), eq("status_code", 200)))
                .getDeletedCount();
        log.info("清空过期数据量 : {}", deletedCount);
        MongoDatabase db = mongoTemplate.getDb();
        Consumer<List<Document>> func = list -> {
            list.stream().map(d -> d.getString("file_path"))
                    .filter(StringUtils::isNotEmpty)
                    .forEach(keySuffix -> {
                        copyObject2OSS(keySuffix, ds);
                    });
        };
        MongoDbFastScan mongoDbFastScan = new MongoDbFastScan(AUCTION_TRACKING_ANSHUO_URL, func, db)
                .batchSize(200).threadNum(100);
        mongoDbFastScan.scan();
    }

    public void copyObject2OSS(String keySuffix, String ds) {
        String sourceKey = project.getSourcePathPrefix() + keySuffix;
        String targetKey = project.getTargetPathprefix() + ds + "/file/" + keySuffix;
        Document d = new Document();
        d.put("file_path", keySuffix);
        d.put("update_time", new Date());
        d.put("ds", ds);
        int statusCode = 500;
        UpdateOptions up = new UpdateOptions().upsert(true).bypassDocumentValidation(true);
        Document id = new Document("_id", md5(keySuffix + ds));
        try {
            CopyObjectResult copyObjectResult = ossClient.copyObject(project.getSourceBucketName(), sourceKey, project.getTargetBucketName(), targetKey);
            statusCode = copyObjectResult.getResponse().getStatusCode();
            d.put("status_code", statusCode);
            log.info("code : {} \n,key : {}", statusCode, targetKey);
        } catch (Exception e) {
            log.error("code : {} \n,key : {}\n,error : {}", statusCode, targetKey, e.getMessage());
            int length = e.getMessage().length();
            d.put("error_message", length > 1000 ? e.getMessage().substring(0, 1000) : e.getMessage());
            mongoTemplate.getCollection("xf_as_upload_oss_error_log")
                    .updateOne(id, new Document("$set", d), up);
        }
        mongoTemplate.getCollection("xf_as_upload_oss_log")
                .updateOne(id, new Document("$set", d), up);
    }

    public void saveObject(String ds) {
        String targetKey = project.getTargetPathprefix() + ds + "/anshuo.txt";
        PutObjectRequest putObjectRequest = new PutObjectRequest(project.getTargetBucketName(), targetKey, new ByteArrayInputStream("".getBytes()));
        ossClient.putObject(putObjectRequest);

    }

    public void saveEmptyObject(String ds) {
        //空文书输出
        List<String> out_tns = Arrays.asList("wenshu_detail_v2", "wenshu_original");
        String ossPre = "anshuo/wenshu/";
        List<String> res = listKeys(project.getTargetBucketName(), ossPre + ds);
        List<String> tns = res.stream()
                .map(key -> key.replace(".json", "")
                        .substring(key.lastIndexOf("/") + 1))
                .distinct()
                .collect(Collectors.toList());

        out_tns.stream()
                .filter(tn -> !tns.contains(tn))
                .forEach(tn -> {
                    String targetKey = ossPre + ds + "/" + tn + ".json";
                    PutObjectRequest putObjectRequest = new PutObjectRequest(project.getTargetBucketName(), targetKey, new ByteArrayInputStream("".getBytes()));
                    ossClient.putObject(putObjectRequest);
                });

    }

    @Override
    public void saveObject(String ds, String targetBucketName, String ossPre) {
        String prcessDate = getPrcessDateV2(ds, 7);
        String pre = ossPre + "/" + prcessDate + "/";
        List<String> keys = listKeys(targetBucketName, pre);
        List<String> filters_keys = keys.stream()
                .filter(k -> tc_tns.contains(getTn(k)))
                .distinct()
                .collect(Collectors.toList());
        String sum = JSON.toJSONString(filters_keys);
        String targetKey = pre + "out_tc.success";
        PutObjectRequest putObjectRequest = new PutObjectRequest(targetBucketName, targetKey, new ByteArrayInputStream(sum.getBytes()));
        ossClient.putObject(putObjectRequest);
    }

    public void saveObjectBase(String ds, TaskRunParam runParam) {
        String prcessDate = getPrcessDateV2(ds, 0);
        String pre = runParam.getOss_pre() + "/" + prcessDate + "/";
        List<String> keys = listKeys(runParam.getBucket_name(), pre, runParam.getDetail_suffix());
        List<String> filters_keys = keys.stream()
                .filter(k -> runParam.getTns_out().contains(getTn(k)))
                .distinct()
                .collect(Collectors.toList());
        String sum = JSON.toJSONString(filters_keys);
        String targetKey = pre + "out.success";
        PutObjectRequest putObjectRequest = new PutObjectRequest(runParam.getBucket_name(), targetKey, new ByteArrayInputStream(sum.getBytes()));
        ossClient.putObject(putObjectRequest);
    }

    /**
     * 安硕工商数据测试
     *
     * @param ds
     * @param targetBucketName
     * @param ossPre
     */
    @Override
    public void saveObjectAnshuo(String ds, String targetBucketName, String ossPre) {
        String prcessDate = getPrcessDateV2(ds, 0);
        anshuo_company_tns.forEach(tn -> {
            String targetSuccessKey = ossPre + "/" + tn + "/" + prcessDate + "/" + "anshuo.txt";
            String targetDataKey = ossPre + "/" + tn + "/" + prcessDate + "/" + tn + ".json";
            if (existsBucketName(targetBucketName, targetDataKey)) {
                PutObjectRequest putObjectRequest = new PutObjectRequest(targetBucketName, targetSuccessKey, new ByteArrayInputStream("".getBytes()));
                ossClient.putObject(putObjectRequest);
            }
        });
    }

    public String getTn(String key) {
        String re = key.substring(0, key.lastIndexOf("/"));
        return re.substring(re.lastIndexOf("/") + 1);
    }
}
